var _DISPLAYS;

// map the DSL standard to JS: https://docs.mpi-internal.com/scmspain/all--lib-form-builder-docs/form-specification/field/
import { pickFieldById, fieldsNamesInOrderOfDefinition } from '../reducer/fields';
import { LocalizationFactory } from './../Standard/Localization/LocalizationFactory';
var FIELDS = {
  TEXT: 'text',
  NUMERIC: 'numeric',
  FIELDSET: 'fieldset',
  PICKER: 'picker',
  MULTIPICKER: 'multipicker'
};
var DISPLAYS = (_DISPLAYS = {}, _DISPLAYS[FIELDS.TEXT] = {
  MULTILINE: 'multiline',
  EMAIL: 'email',
  PHONE: 'phone',
  // not part of the DSL standard
  TEXT: 'text',
  // not part of the DSL standard
  NUMERIC: 'numeric',
  // not part of the DSL standard
  DEFAULT: ''
}, _DISPLAYS[FIELDS.NUMERIC] = {
  MONEY: 'money',
  SURFACE: 'surface',
  DEFAULT: ''
}, _DISPLAYS[FIELDS.PICKER] = {
  SWITCH: 'switch',
  AUTOCOMPLETE: 'autocomplete',
  DROPDOWN: 'dropdown',
  RADIO: 'radio',
  CHECKBOX: 'checkbox',
  BUTTON_INLINE: 'button-inline',
  STEPPER: 'stepper'
}, _DISPLAYS[FIELDS.MULTIPICKER] = {
  BUTTON: 'button'
}, _DISPLAYS);
var CONSTRAINTS = {
  notnull: 'valueMissing',
  pattern: 'patternMismatch',
  maxlength: 'tooLong',
  minlength: 'tooShort',
  max: 'rangeOverflow',
  min: 'rangeUnderflow'
};

var checkConstraintsFromField = function checkConstraintsFromField(field, locale) {
  var errorMessages = [];
  var constraints = field.constraints || [];
  var elementNode = document.getElementById(field.id);
  var elementValidity = elementNode == null ? void 0 : elementNode.validity; // if element has no validity is not a form element
  // if element has not constraints there is no need to validate

  if (elementValidity && !!constraints) {
    errorMessages = constraints.filter(function (constraint) {
      // for each constraint property defined in the field
      var validityRule = CONSTRAINTS[Object.keys(constraint == null ? void 0 : constraint.property)[0]]; // if true, the constraint is not satisfied, so add the constraint to errors

      if (elementValidity[validityRule]) return true;
      return false;
    }) // return
    .map(function (constraint) {
      return constraint.message;
    });
  } // custom validation: minlength constraint in html textarea is not working as expected, need to handle it manually


  if ( // if is a field text and display multiline
  field.type === FIELDS.TEXT && field.display === DISPLAYS[FIELDS.TEXT].MULTILINE && // if element validity is tooShort means that it is working as expected and it is not required manual validation
  !elementValidity.tooShort) {
    var _field$constraints, _textAreaHasMinLength;

    var textAreaHasMinLength = (_field$constraints = field.constraints) == null ? void 0 : _field$constraints.find(function (constraint) {
      var _constraint$property;

      return (_constraint$property = constraint.property) == null ? void 0 : _constraint$property.minlength;
    });
    var textAreaValue = elementNode == null ? void 0 : elementNode.value;
    if ((textAreaValue == null ? void 0 : textAreaValue.length) < parseInt(textAreaHasMinLength == null ? void 0 : (_textAreaHasMinLength = textAreaHasMinLength.property) == null ? void 0 : _textAreaHasMinLength.minlength)) errorMessages = [].concat(errorMessages, [textAreaHasMinLength.message]);
  } // custom validation: since SUI select and autocomplete add disabled and readOnly props to input the default validity function marks allways as valid the field


  if (field.type === FIELDS.PICKER && (field.display === DISPLAYS[FIELDS.PICKER].AUTOCOMPLETE || field.display === DISPLAYS[FIELDS.PICKER].DROPDOWN || field.display === '')) {
    var _field$constraints2;

    var pickerValue = elementNode == null ? void 0 : elementNode.value;
    var pickerShouldNotBeNullConstraint = (_field$constraints2 = field.constraints) == null ? void 0 : _field$constraints2.find(function (constraint) {
      var _constraint$property2;

      return ((_constraint$property2 = constraint.property) == null ? void 0 : _constraint$property2.notnull) === '';
    });

    if (pickerShouldNotBeNullConstraint && !pickerValue) {
      errorMessages = [pickerShouldNotBeNullConstraint.message].concat(errorMessages);
    }
  } // custom validation: pattern constraint in html input (type=checkbox) is not natively supported, need to handle it manually


  if (field.type === FIELDS.PICKER && field.display === DISPLAYS[FIELDS.PICKER].CHECKBOX) {
    var _field$constraints3;

    var checkboxValue = (elementNode == null ? void 0 : elementNode.value) === 'true';
    var checkboxShouldBeTrueConstraint = (_field$constraints3 = field.constraints) == null ? void 0 : _field$constraints3.find(function (constraint) {
      var _constraint$property3;

      return ((_constraint$property3 = constraint.property) == null ? void 0 : _constraint$property3.pattern) === '^true$';
    }); // if should be true according to field, and is false show error message

    if (checkboxShouldBeTrueConstraint && !checkboxValue) errorMessages = [checkboxShouldBeTrueConstraint.message].concat(errorMessages);
  } // cutom validation: min html attributes in input type text are not nativelly supported


  if ( // if is a field text and display text or default
  field.type === FIELDS.TEXT && field.display === DISPLAYS[FIELDS.TEXT].TEXT || field.display === DISPLAYS[FIELDS.TEXT].DEFAULT) {
    var _field$constraints4, _inputHasMin$property;

    var inputHasMin = (_field$constraints4 = field.constraints) == null ? void 0 : _field$constraints4.find(function (constraint) {
      var _constraint$property4;

      return (_constraint$property4 = constraint.property) == null ? void 0 : _constraint$property4.min;
    });
    var localization = LocalizationFactory({
      value: elementNode == null ? void 0 : elementNode.value,
      locale: locale
    });
    var inputValueAsNumber = localization.fromStringToLocaleFloat();
    if (inputValueAsNumber < parseFloat(inputHasMin == null ? void 0 : (_inputHasMin$property = inputHasMin.property) == null ? void 0 : _inputHasMin$property.min)) errorMessages = [].concat(errorMessages, [inputHasMin.message]);
  } // cutom validation: max html attributes in input type text are not nativelly supported


  if ( // if is a field text and display text or default
  field.type === FIELDS.TEXT && field.display === DISPLAYS[FIELDS.TEXT].TEXT || field.display === DISPLAYS[FIELDS.TEXT].DEFAULT) {
    var _field$constraints5, _inputHasMax$property;

    var inputHasMax = (_field$constraints5 = field.constraints) == null ? void 0 : _field$constraints5.find(function (constraint) {
      var _constraint$property5;

      return (_constraint$property5 = constraint.property) == null ? void 0 : _constraint$property5.max;
    });

    var _localization = LocalizationFactory({
      value: elementNode == null ? void 0 : elementNode.value,
      locale: locale
    });

    var _inputValueAsNumber = _localization.fromStringToLocaleFloat();

    if (_inputValueAsNumber > parseFloat(inputHasMax == null ? void 0 : (_inputHasMax$property = inputHasMax.property) == null ? void 0 : _inputHasMax$property.max)) {
      errorMessages = [].concat(errorMessages, [inputHasMax.message]);
    }
  }

  return errorMessages;
};

var checkConstraintsFactory = function checkConstraintsFactory(json, locale) {
  return function (_ref) {
    var fieldID = _ref["for"],
        all = _ref.all;
    var fieldsToValidate = [];

    if (all && fieldID) {
      window.console.warn('[form/builder]: checkConstraintsFactory: both modes validate all fields and validate a concrete field are not compatible, please use one of them');
    } else if (all) {
      var _json$form;

      fieldsToValidate = fieldsNamesInOrderOfDefinition(json == null ? void 0 : (_json$form = json.form) == null ? void 0 : _json$form.fields);
    } else if (fieldID) {
      fieldsToValidate = [fieldID];
    } else {
      window.console.warn('[form/builder]: checkConstraintsFactory: Specify if you want to validate a specific field or all the fields');
    }

    var fieldsWithErrors = {};
    fieldsToValidate.forEach(function (fieldId) {
      var field = pickFieldById(json.form.fields, fieldId);

      if (!field.hidden) {
        fieldsWithErrors[field.id] = checkConstraintsFromField(field, locale);
      } else {
        fieldsWithErrors[field.id] = [];
      }
    });
    return fieldsWithErrors;
  };
};

export { FIELDS, DISPLAYS, CONSTRAINTS, checkConstraintsFactory };